<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb23000BalanceteGerencialFactory extends \Teknisa\CTB\Controllers\TeknisaCrud {

    protected $connection;
    
    const PROCEDURE_GERA_BALANCETE = "GERA_BALANCETE_GERENCIAL";

    public function __construct(EntityManager $entityManager, $environment) {
        $this->connection  = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    public function buscaDadosBalancete($cdCentCust, $nrctbbalgertmp, $minCont) {
                
        $sql = "select * from (
                    SELECT formata_cdcontctbl(C.CDCONTCTBLT) as CDCONTCTBLT, CB.CDCONTCTBL, CB.NMCONTCTBL, C.CDCENTCUSTT, (VRDEBITOT+VRCREDITOT) AS VRSALDATUAT, C.NRCTBBALGERTMP
                        FROM CTBBALGERTMP C, 
                        CONTCTBL CB, FILIAL F, CENTCUST CT, EMPRESA E
                        WHERE C.NRCTBBALGERTMP = :P_NRCTBBALGERTMP
                        AND C.CDCONTCTBLT = CB.CDCONTCTBL
                        AND C.CDFILIALT = F.CDFILIAL
                        AND C.CDCENTCUSTT = CT.CDCENTCUST
                        AND F.CDEMPRESA = E.CDEMPRESA
                        AND (C.CDCONTCTBLT > :P_MINCONT
                        OR :P_MINCONTT = 'T')
                        AND C.CDCENTCUSTT IN $cdCentCust
                        GROUP BY C.CDCONTCTBLT, CB.CDCONTCTBL, CB.NMCONTCTBL, C.CDCENTCUSTT, (VRDEBITOT+VRCREDITOT), C.NRCTBBALGERTMP
                    )
                    pivot 
                    (
                        max(VRSALDATUAT)
                        for CDCENTCUSTT in $cdCentCust
                    )
                    ORDER BY CDCONTCTBL";
        
        $params = array(
            "P_NRCTBBALGERTMP" => $nrctbbalgertmp,
            "P_MINCONT" => $minCont,
            "P_MINCONTT" => $minCont
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getAllCdCentCust($cdFilial, $centCust = "'T'") {
        if($centCust != "'T'") {
            $sql = "SELECT DISTINCT
              CF.CDCENTCUST,
              C.NMCENTCUST
            FROM CENTCUSTFIL CF
            JOIN CENTCUST C
            ON CF.CDCENTCUST = C.CDCENTCUST
            AND CF.NRORG     = C.NRORG
            JOIN FILIAL F
            ON CF.CDFILIAL  = F.CDFILIAL
            AND CF.NRORG    = F.NRORG
            AND CF.NRORG    = :NRORG
            AND CF.CDFILIAL IN $cdFilial
            AND (CF.CDCENTCUST IN $centCust) 
            ORDER BY CF.CDCENTCUST";
        }
        else {
           $sql = "SELECT DISTINCT
              CF.CDCENTCUST,
              C.NMCENTCUST
            FROM CENTCUSTFIL CF
            JOIN CENTCUST C
            ON CF.CDCENTCUST = C.CDCENTCUST
            AND CF.NRORG     = C.NRORG
            JOIN FILIAL F
            ON CF.CDFILIAL  = F.CDFILIAL
            AND CF.NRORG    = F.NRORG
            AND CF.NRORG    = :NRORG
            AND CF.CDFILIAL IN $cdFilial
            ORDER BY CF.CDCENTCUST";
        }
        
        $params = array(
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getParamGeral() {
        $sql = "SELECT * FROM PARAMGERAL WHERE NRORG = :NRORG";
        
        $params = array(
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll()[0];
    }
    
    public function geraDadosBalancete($cdFilial, $cdEmpresa, $cdCentCustGera, 
            $dataIni, $dataFin, $dataSaldo, $contaIni, $contaFin, $cdNvCont) {
        try {
            $nrBalancete = '';
            $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_BALANCETE);
            $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT,     $cdEmpresa,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CCUST',           Param::PARAM_INPUT,     $cdCentCustGera,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT,     $cdFilial,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',       Param::PARAM_INPUT,     $dataIni,           Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL',         Param::PARAM_INPUT,     $dataFin,           Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTSALDO',         Param::PARAM_INPUT,     $dataSaldo,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCONTAINI',      Param::PARAM_INPUT,     $contaIni,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCONTAFIN',      Param::PARAM_INPUT,     $contaFin,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NIVEL',           Param::PARAM_INPUT,     $cdNvCont,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRCTBBALGERTMP', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 20));
            $outPut   = $procedure->execute();
            return $outPut['P_NRCTBBALGERTMP'];
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

}